package main

import (
	"io"
	"path/filepath"
	"strings"
)

type PackageErrors []string

func (eh *PackageErrors) Add(pkg *Package, reason string) {
	*eh = append(*eh, pkg.Name()+": "+reason)
}

func (eh *PackageErrors) SkipPkgAndDependees(pkg *Package, reason string) {
	eh.Add(pkg, reason)
	eh.skipDependees(pkg)
}

func (eh *PackageErrors) CheckDownloads(head *Package, tmpDir string) {
	head.Iterate(func(pkg *Package) {
		md5sums := pkg.Md5sums()
		for i, DownloadName := range pkg.DownloadNames() {
			fullPath := filepath.Join(tmpDir, DownloadName)
			switch {
			case !Exists(fullPath):
				eh.Add(pkg, fullPath+": file not found")
			case !IsMD5SumMatch(md5sums[i], fullPath):
				eh.Add(pkg, fullPath+": MD5Sum Mismatch. Run with -"+MD5_FLAG+" to disable MD5Sum Check")
			default:
				continue
			}
			eh.skipDependees(pkg)
		}
	})
}

func (eh *PackageErrors) skipDependees(pkgToDelete *Package) {
	for _, dependee := range pkgToDelete.Dependees() {
		if dependee.IsInList() {
			eh.Add(dependee, "Depends on "+pkgToDelete.Name())
			eh.skipDependees(dependee)
		}
	}
	pkgToDelete.Del()
}

func (eh *PackageErrors) Final(w io.Writer) {
	s := "All packages installed successfully"
	if len(*eh) != 0 {
		s = Banner("Skipped Packages", strings.Join(*eh, "\n"))
	}
	w.Write([]byte(s + "\n"))
}
