// Description: Structs to execute external commands.
package main

import (
	"io"
	"log"
	"os/exec"
	"strings"
)

type Executor struct {
	stdout, stderr io.Writer
	logger         *log.Logger
}

func (e *Executor) Run(full_cmd string) error {
	e.logger.Println(full_cmd)
	split := strings.Split(full_cmd, " ")
	cmd := exec.Command(split[0], split[1:]...)
	cmd.Stdout = e.stdout
	cmd.Stderr = e.stderr
	status := cmd.Run()
	if status != nil {
		e.logger.Println(status)
	}
	return status
}

func (e *Executor) PanicOnFail(full_cmd string) {
	if err := e.Run(full_cmd); err != nil {
		panic(err)
	}
}

type Pager struct {
	executor *Executor
	cmd      string
}

func NewPager(exec *Executor) *Pager {
	return &Pager{exec, SetEnvIfNotSet("PAGER", FALLBACK_PAGER)}
}

func (p *Pager) Display(file string) {
	p.executor.PanicOnFail(p.cmd + " " + file)
}
