package main

import (
	"log"
	"os"
	"strings"
	"testing"
)

func TestNewPager(t *testing.T) {
	os.Setenv("PAGER", "more")
	devnull, _ := os.Open(os.DevNull)
	defer devnull.Close()
	exec := &Executor{devnull, devnull, log.New(devnull, "", 0)}
	pager := NewPager(exec)
	if pager.cmd != "more" {
		t.Error("NewPager() returned an incorrect command")
	}
	os.Setenv("PAGER", "")
	pager = NewPager(exec)
	if pager.cmd != FALLBACK_PAGER {
		t.Error("NewPager() returned an incorrect command")
	}
	if pager.executor.stdout != devnull {
		t.Error("NewPager() returned an incorrect stdout")
	}
	if pager.executor.stderr != devnull {
		t.Error("NewPager() returned an incorrect stderr")
	}
}

func TestRun(t *testing.T) {
	dummyStdout, dummyStderr := strings.Builder{}, strings.Builder{}
	devnull, _ := os.Open(os.DevNull)
	defer devnull.Close()
	logger := log.New(devnull, "", 0)
	executor := Executor{&dummyStdout, &dummyStderr, logger}
	err := executor.Run("echo foo")
	if err != nil {
		t.Error("Run() returned an error")
	}
	if dummyStdout.String() != "foo\n" {
		t.Error("Run() returned an incorrect stdout")
	}
	if dummyStderr.String() != "" {
		t.Error("Run() returned an incorrect stderr")
	}
}

func TestPanicOnFail(t *testing.T) {
	dummyStdout, dummyStderr := strings.Builder{}, strings.Builder{}
	devnull, _ := os.Open(os.DevNull)
	defer devnull.Close()
	logger := log.New(devnull, "", 0)
	executor := Executor{&dummyStdout, &dummyStderr, logger}
	defer func() {
		if r := recover(); r == nil {
			t.Error("PanicOnFail() did not panic")
		}
	}()
	executor.PanicOnFail("false")
}
