package main

import (
	"os"
	"strings"
	"testing"
)

func equalStringSlices(a, b []string) bool {
	if len(a) != len(b) {
		return false
	}
	for i := range a {
		if a[i] != b[i] {
			return false
		}
	}
	return true
}

func containsAll(str string, subs ...string) bool {
	for _, s := range subs {
		if !strings.Contains(str, s) {
			return false
		}
	}
	return true
}

func mockPkg() *Package {
	var (
		head       = &Package{}
		tail       = head
		prev, next = &Package{name: "prev"}, &Package{name: "next"}
		p          = &Package{
			name:      "name",
			path:      "category",
			dependees: []*Package{prev, next},
		}
	)
	tail = prev.Append(tail)
	tail = p.Append(tail)
	tail = next.Append(tail)
	return p
}

func createPkgTestfiles(p *Package, t *testing.T) {
	createTestfileArgs(p, t,
		"http://example.com/foo.tar.gz", "1234567890",
		"", "",
		"%README% dependee1",
	)
}

func createTestfileArgs(p *Package, t *testing.T, download, md5sum, download64, md5sum64, requires string) {
	tdir := t.TempDir()
	p.path = tdir
	Mkdir(p.Path())
	createTestFile(p.SlackBuild(), "")
	createTestFile(p.Readme(), "README file\n")
	createTestFile(p.InfoFile(), "PRGNAM=\""+p.name+"\"\n"+
		"VERSION=\"1.0\"\n"+
		"HOMEPAGE=\"http://example.com\"\n"+
		"DOWNLOAD=\""+download+"\"\n"+
		"MD5SUM=\""+md5sum+"\"\n"+
		"DOWNLOAD_x86_64=\""+download64+"\"\n"+
		"MD5SUM_x86_64=\""+md5sum64+"\"\n"+
		"REQUIRES=\""+requires+"\"\n"+
		"MAINTAINER=\"John Doe\"\n"+
		"EMAIL=\"mail[at]provider[dot]com\"\n",
	)
	createTestFile(p.SlackDesc(),
		"# HOW TO EDIT THIS FILE:\n"+
			"# The \"handy ruler\" below makes it easier to edit a package description.  Line\n"+
			"# up the first '|' above the ':' following the base package name, and the '|'\n"+
			"# on the right side marks the last column you can put a character in.  You must\n"+
			"# make exactly 11 lines for the formatting to be correct.  It's also\n"+
			"# customary to leave one space after the ':'.\n"+
			"\n"+
			"      |-----handy-ruler------------------------------------------------------|\n"+
			"name: name Description\n"+
			"name:\n"+
			"name:\n"+
			"name:\n"+
			"name:\n"+
			"name:\n"+
			"name:\n"+
			"name:\n"+
			"name:\n"+
			"name:\n"+
			"name:\n",
	)
}

// This helper modifies the .info file contents so that it has the given REQUIRES line.
// A typical info file creation might look like this:
func createPkgTestfilesWithRequires(p *Package, t *testing.T, requiresLine string) {
	createTestfileArgs(p, t,
		"http://example.com/foo.tar.gz", "1234567890",
		"", "",
		requiresLine,
	)
}

// mustMkdir is a helper that creates a directory or fails the test.
func mustMkdir(t *testing.T, path string) {
	t.Helper()
	if err := os.Mkdir(path, 0o755); err != nil {
		t.Fatalf("failed to create directory %q: %v", path, err)
	}
}

// mustWriteFile is a helper that writes content to a file or fails the test.
func mustWriteFile(t *testing.T, path, content string) {
	t.Helper()
	if err := os.WriteFile(path, []byte(content), 0o644); err != nil {
		t.Fatalf("failed to write file %q: %v", path, err)
	}
}
