// Description: IO functions not fitting in any other file.
package main

import (
	"bufio"
	"crypto/md5"
	"encoding/hex"
	"io"
	"strings"
	"syscall"
)

// A banner which can be printed to raise attention to a message
func Banner(heading, msg string) string {
	nsep := 80 - len(heading) - 2
	return "\n" + sep(nsep/2) + " " + heading + " " + sep(nsep/2) +
		sep(nsep%2) + "\n" + msg + "\n" + sep(80)
}

func sep(nlen int) string { return strings.Repeat("-", nlen) }

func ReadPkgNamesFromFile(filename string) []string {
	f := open(filename)
	defer f.Close()
	return StripCommentsAndTokenize(f)
}

func StripCommentsAndTokenize(r io.Reader) (tokens []string) {
	scanner := bufio.NewScanner(r)
	for scanner.Scan() {
		line := scanner.Text()
		commentIndex := strings.Index(line, "#")
		if commentIndex != -1 {
			line = line[:commentIndex]
		}
		tokens = append(tokens, strings.Fields(line)...)
	}
	return
}

func PkgNamesAndBuildOptions(s []string) ([]string, map[string][]string) {
	buildOptions := make(map[string][]string)
	pkgNames := make([]string, 0)
	for i := 0; i < len(s); {
		name := s[i]
		pkgNames = append(pkgNames, name)
		for i++; i < len(s) && strings.Contains(s[i], "="); i++ {
			buildOptions[name] = append(buildOptions[name], s[i])
		}
	}
	return pkgNames, buildOptions
}

func GetKernelVersion() string {
	var (
		uname syscall.Utsname
		i     int
		s     [65]byte
	)
	handleOSError(syscall.Uname(&uname))
	for ; uname.Release[i] != 0; i++ {
		s[i] = byte(uname.Release[i])
	}
	return string(s[:i])
}

func IsMD5SumMatch(md5sum, filename string) bool {
	f := open(filename)
	defer f.Close()
	return md5sum == hex.EncodeToString(md5Sum(f))
}

func md5Sum(r io.Reader) []byte {
	hasher := md5.New()
	Copy(r, hasher)
	return hasher.Sum(nil)
}

func PromptUser(w *bufio.Writer, r io.Reader, prompt string) bool {
	w.WriteString(prompt + " [y/N]: ")
	w.Flush()
	b := ReadChar(r)
	for c := b; c != '\n'; { // consume until newline
		c = ReadChar(r)
	}
	return b == 'y' || b == 'Y'
}

func Join(s []string) string { return strings.Join(s, " ") }
