// Description: This file contains the wrapper functions for the os and sys package.

package main

import (
	"io"
	"os"
	"time"
)

func Mkdir(path string)               { handleOSError(os.MkdirAll(path, 0755)) }
func Symlink(oldname, newname string) { handleOSError(os.Symlink(oldname, newname)) }

func SetEnvIfNotSet(envvar, fallback string) string {
	value := os.Getenv(envvar)
	if value == "" {
		os.Setenv(envvar, fallback)
		return fallback
	}
	return value
}

func Copy(src io.Reader, dest io.Writer) {
	_, err := io.Copy(dest, src)
	handleOSError(err)
}

func ReadChar(r io.Reader) byte {
	char := make([]byte, 1)
	_, err := r.Read(char)
	handleOSError(err)
	return char[0]
}

func Create(file string) *os.File {
	f, err := os.Create(file)
	handleOSError(err)
	return f
}

func open(file string) *os.File {
	f, err := os.Open(file)
	handleOSError(err)
	return f
}

func ReadDir(path string) []os.DirEntry {
	files, err := os.ReadDir(path)
	handleOSError(err)
	return files
}

func ReadFile(path string) []byte {
	data, err := os.ReadFile(path)
	handleOSError(err)
	return data
}

func handleOSError(err error) {
	if err != nil {
		panic(err)
	}
}

func Exists(path string) bool {
	_, err := os.Stat(path)
	return !os.IsNotExist(err)
}

func IsExecutable(path string) bool {
	info, err := os.Stat(path)
	return err == nil && info.Mode().IsRegular() && info.Mode().Perm()&0111 != 0
}

func ModTime(file os.DirEntry) time.Time {
	info, err := file.Info()
	handleOSError(err)
	return info.ModTime()
}
